@@ -7,7 +7,6 @@ use num_bigint::BigUint;
 use pyo3::{prelude::*, types::PyTuple};
 use rr_util::{
     atr,
-    compact_data::U8Set,
     rearrange_spec::RInnerInts,
     tensor_util::{IndexError, IndexTensor, TensorIndexSync},
     tu8v,
@@ -61,7 +60,8 @@ circuit_node_extra_impl!(Einsum);
 
 impl CircuitNodeComputeInfoImpl for Einsum {
     fn compute_shape(&self) -> Shape {
-        self.info().shape.clone()
+        let map = self.shape_map().unwrap();
+        self.out_axes.iter().map(|a| *map.get(a).unwrap()).collect()
     }
 }
 impl CircuitNodeHashWithChildren for Einsum {
@@ -197,7 +197,7 @@ impl Einsum {
     #[apply(new_rc_unwrap)]
     pub fn try_new(args: EinsumArgs, out_axes: EinsumAxes, name: Option<String>) -> Result<Self> {
         // TODO: reduce axes nums!
-        let mut out = Self {
+        let out = Self {
             args,
             out_axes,
             name,
@@ -206,19 +206,18 @@ impl Einsum {
 
         let map = out.shape_map()?; // catch errors
 
-        out.info.shape = out
-            .out_axes
-            .iter()
-            .map(|a| map.get(a).cloned())
-            .collect::<Option<Shape>>()
-            .ok_or_else(|| {
-                anyhow::Error::from(ConstructError::EinsumOutputNotSubset {
+        for a in &out.out_axes {
+            if !map.contains_key(a) {
+                return Err(ConstructError::EinsumOutputNotSubset {
                     circuit_name: out.name.clone(),
                     all_input_axes: map.into_keys().collect(),
                     output_axes: out.out_axes.clone(),
-                })
-            })?;
+                }
+                .into());
+            }
+        }
 
+        // could reuse hashmap if helped with perf
         out.initial_init_info()
     }
 
@@ -488,7 +487,7 @@ impl Einsum {
         .unwrap()
     }
 
-    pub fn reduced_axes(&self) -> U8Set {
+    pub fn reduced_axes(&self) -> HashSet<u8> {
         self.input_axes()
             .flatten()
             .filter(|i| !self.out_axes.contains(i))
