@@ -12,7 +12,6 @@ use pyo3::{
 use rand::Rng;
 use regex::Regex;
 use rr_util::{
-    name::Name,
     py_types::{assert_tensors_close, ExtraPySelfOps, PyShape, Tensor, PY_UTILS},
     pycall, python_error_exception, simple_from,
     tensor_util::{
@@ -42,8 +41,8 @@ macro_rules! gf_gen {
 
         $(
         #[pyfunction]
-        pub fn $name(circuit: CircuitRc, name: Option<Name>) -> Result<GeneralFunction> {
-            GeneralFunction::new_by_name(vec![circuit], stringify!($name).into(), name)
+        pub fn $name(circuit: CircuitRc, name: Option<String>) -> Result<GeneralFunction> {
+            GeneralFunction::new_by_name(vec![circuit], stringify!($name).to_owned(), name)
         }
         )*
 
@@ -81,14 +80,14 @@ gf_gen!(
     (last_dim_size, 0, 1),
 );
 
-static SPECS: GILLazy<HashMap<Name, GeneralFunctionSpec>> = GILLazy::new(|| {
+static SPECS: GILLazy<HashMap<String, GeneralFunctionSpec>> = GILLazy::new(|| {
     BASIC_SPEC_ITEMS
         .iter()
         .cloned()
         .map(|(name, num_non_batchable_output_dims, removed_from_end)| {
-            let name = name.into();
+            let name = name.to_owned();
             (
-                name,
+                name.clone(),
                 GeneralFunctionSimpleSpec {
                     name,
                     num_non_batchable_output_dims,
@@ -111,7 +110,7 @@ pub trait SpecTrait: fmt::Debug + ToPyObject {
     fn get_shape_info(&self, shapes: &[Shape]) -> Result<GeneralFunctionShapeInfo>;
     // fn has_jacobian(&self) -> Result<bool>;
     // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>>;
-    fn name(&self) -> &'static str;
+    fn name(&self) -> String;
     fn is_official(&self) -> bool {
         false
     }
@@ -161,7 +160,7 @@ impl GeneralFunctionShapeInfo {
 #[derive(Debug, Clone)]
 pub struct GeneralFunctionSimpleSpec {
     #[pyo3(get)]
-    name: Name,
+    name: String,
     #[pyo3(get)]
     num_non_batchable_output_dims: u8,
     #[pyo3(get)]
@@ -185,7 +184,7 @@ pub fn get_shape_info_simple(
     removed_from_end: Option<u8>,
 ) -> Result<GeneralFunctionShapeInfo> {
     GeneralFunctionSimpleSpec {
-        name: "".into(),
+        name: "".to_owned(),
         num_non_batchable_output_dims: num_non_batchable_output_dims.unwrap_or(0),
         removed_from_end: removed_from_end.unwrap_or(0),
     }
@@ -195,7 +194,7 @@ pub fn get_shape_info_simple(
 #[pymethods]
 impl GeneralFunctionSimpleSpec {
     fn get_function(&self) -> PyObject {
-        PY_UTILS.generalfunctions[&self.name.string()].clone()
+        PY_UTILS.generalfunctions[&self.name].clone()
     }
 }
 
@@ -253,14 +252,14 @@ impl SpecTrait for GeneralFunctionSimpleSpec {
     // }
     // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>> {}
 
-    fn name(&self) -> &'static str {
-        self.name.into()
+    fn name(&self) -> String {
+        self.name.clone()
     }
     fn is_official(&self) -> bool {
         true
     }
     fn serialize(&self) -> Result<Option<String>> {
-        Ok(Some(self.name().into()))
+        Ok(Some(self.name.clone()))
     }
 }
 
@@ -396,20 +395,19 @@ impl SpecTrait for GeneralFunctionIndexSpec {
         }
         Some(Ok(device_dtypes[0]))
     }
-    fn name(&self) -> &'static str {
-        Name::new(&format!(
+    fn name(&self) -> String {
+        format!(
             "gen_index_at_{}{}{}",
             self.index_dim,
             if self.batch_x { "_batch_x" } else { "" },
             if self.check_index_ints { "_c" } else { "" }
-        ))
-        .into()
+        )
     }
     fn is_official(&self) -> bool {
         true
     }
     fn serialize(&self) -> Result<Option<String>> {
-        Ok(Some(self.name().into()))
+        Ok(Some(self.name()))
     }
 }
 
@@ -484,15 +482,14 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
     // }
     // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>> {}
 
-    fn name(&self) -> &'static str {
-        Name::new(&format!(
+    fn name(&self) -> String {
+        format!(
             "cast_{:?}_{:?}",
             self.input_required_compatibility, self.output
-        ))
-        .into()
+        )
     }
     fn serialize(&self) -> Result<Option<String>> {
-        Ok(Some(self.name().to_owned()))
+        Ok(Some(self.name()))
     }
 }
 
@@ -500,7 +497,7 @@ impl SpecTrait for GeneralFunctionSetDDSpec {
 pub struct PyWrap {
     ob: PyObject,
     hash: HashBytes,
-    name: Name,
+    name: String,
     path: Option<String>,
 }
 
@@ -565,7 +562,7 @@ impl<'source> pyo3::FromPyObject<'source> for Box<PyWrap> {
         }
 
         Ok(Box::new(PyWrap {
-            name: name.into(),
+            name,
             path,
             ob,
             hash: *hasher.finalize().as_bytes(),
@@ -639,8 +636,8 @@ impl SpecTrait for Box<PyWrap> {
         }
         return resulty.unwrap().map(|x| Ok(x));
     }
-    fn name(&self) -> &'static str {
-        self.name.into()
+    fn name(&self) -> String {
+        self.name.clone()
     }
     fn serialize(&self) -> Result<Option<String>> {
         match &self.path {
@@ -702,7 +699,7 @@ impl SpecTrait for GeneralFunctionSpec {
     }
     // fn has_jacobian(&self) -> Result<bool>;
     // fn get_jacobians(&self, func: &GeneralFunction) -> Result<Option<Vec<Circuit>>>;
-    fn name(&self) -> &'static str {
+    fn name(&self) -> String {
         self.as_trait_obj().name()
     }
     fn is_official(&self) -> bool {
@@ -721,6 +718,7 @@ pub struct GeneralFunction {
     #[pyo3(get)]
     pub spec: GeneralFunctionSpec,
     info: CachedCircuitInfo,
+    name: Option<String>,
     #[pyo3(get)]
     pub num_non_batchable_output_dims: u8,
     pub input_batchability: Vec<bool>,
@@ -731,7 +729,7 @@ impl GeneralFunction {
     pub fn try_new(
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> Result<Self> {
         let shapes = nodes
             .iter()
@@ -756,12 +754,12 @@ impl GeneralFunction {
         let mut out = Self {
             nodes,
             spec,
+            name,
             info: Default::default(),
             num_non_batchable_output_dims,
             input_batchability,
         };
         out.info.shape = shape; // set shape so we only compute once
-        out.info.name = name;
         out.initial_init_info()
     }
 
@@ -812,7 +810,7 @@ impl CircuitNode for GeneralFunction {
                 .map(move |(i, circ)| f(i, circ.clone()))
                 .collect::<Result<Vec<_>, _>>()?,
             self.spec.clone(),
-            self.info().name,
+            self.name.clone(),
         )
     }
 
@@ -842,22 +840,21 @@ impl CircuitNode for GeneralFunction {
 impl CircuitNodeAutoName for GeneralFunction {
     const PRIORITY: OperatorPriority = OperatorPriority::Function {};
 
-    fn auto_name(&self) -> Option<Name> {
+    fn auto_name(&self) -> Option<String> {
         // Never any parenthesis, so we don't even care checking if we need to add some
         // but check out children_names_with_maybe_paren if you change the syntax of the general function
-        if self.children().any(|x| x.info().name.is_none()) {
+        if self.children().any(|x| x.name().is_none()) {
             None
         } else {
             Some(
-                (self.spec.name().to_owned()
+                self.spec.name().clone()
                     + "("
                     + &self
                         .children()
-                        .map(|x| Self::shorten_child_name(x.info().name.unwrap().str()))
+                        .map(|x| Self::shorten_child_name(x.name().unwrap()))
                         .collect::<Vec<String>>()
                         .join(", ")
-                    + ")")
-                    .into(),
+                    + ")",
             )
         }
     }
@@ -870,7 +867,7 @@ impl GeneralFunction {
     fn new_py(
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> PyResult<PyClassInitializer<GeneralFunction>> {
         let out = GeneralFunction::try_new(nodes, spec, name)?;
 
@@ -882,23 +879,27 @@ impl GeneralFunction {
     pub fn new_from_parse(
         nodes: Vec<CircuitRc>,
         parse_string: String,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> Result<Self> {
         if parse_string.contains(" at ") {
             let mut split = parse_string.splitn(2, " at ");
             let name = convert_op_str_to_format_err(split.next(), &parse_string)?;
             let path = convert_op_str_to_format_err(split.next(), &parse_string)?;
 
-            Self::new_by_path(nodes, path, Some(name.into()))
+            Self::new_by_path(nodes, path, Some(name))
         } else {
-            Self::new_by_name(nodes, parse_string.into(), name)
+            Self::new_by_name(nodes, parse_string, name)
         }
     }
 
     #[staticmethod]
     #[args(nodes = "*", spec_name, name = "None")]
-    pub fn new_by_name(nodes: Vec<CircuitRc>, spec_name: Name, name: Option<Name>) -> Result<Self> {
-        Self::new_by_name_op(nodes, spec_name, name)?
+    pub fn new_by_name(
+        nodes: Vec<CircuitRc>,
+        spec_name: String,
+        name: Option<String>,
+    ) -> Result<Self> {
+        Self::new_by_name_op(nodes, spec_name.clone(), name)?
             .ok_or(ConstructError::UnknownGeneralFunction { spec_name }.into())
     }
 
@@ -906,8 +907,8 @@ impl GeneralFunction {
     #[args(nodes = "*", spec_name, name = "None")]
     pub fn new_by_name_op(
         nodes: Vec<CircuitRc>,
-        spec_name: Name,
-        name: Option<Name>,
+        spec_name: String,
+        name: Option<String>,
     ) -> Result<Option<Self>> {
         // parse index case
         static RE: Lazy<Regex> =
@@ -938,7 +939,7 @@ impl GeneralFunction {
 
     #[staticmethod]
     #[args(nodes = "*", path, name = "None")]
-    pub fn new_by_path(nodes: Vec<CircuitRc>, path: String, name: Option<Name>) -> Result<Self> {
+    pub fn new_by_path(nodes: Vec<CircuitRc>, path: String, name: Option<String>) -> Result<Self> {
         let mut split = path.splitn(2, ":");
         let file_path = convert_op_str_to_format_err(split.next(), &path)?;
         let spec_name = convert_op_str_to_format_err(split.next(), &path)?;
@@ -976,7 +977,7 @@ impl GeneralFunction {
         index_dim: i64,
         batch_x: bool,
         check_index_ints: bool,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> Result<Self> {
         let spec = GeneralFunctionIndexSpec {
             index_dim,
@@ -996,7 +997,7 @@ impl GeneralFunction {
         x: CircuitRc,
         input_required_compatibility: TorchDeviceDtypeOp,
         output: TorchDeviceDtypeOp,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> Result<Self> {
         let spec = GeneralFunctionSetDDSpec {
             input_required_compatibility,
@@ -1419,7 +1420,7 @@ fn test_simple_general_function() -> Result<()> {
     for num_non_batchable_output_dims in 0..2 {
         for removed_from_end in 0..2 {
             let spec = GeneralFunctionSimpleSpec {
-                name: "".into(),
+                name: "".to_owned(),
                 num_non_batchable_output_dims,
                 removed_from_end,
             }
