@@ -13,7 +13,6 @@ use rr_util::{
     cached_lambda, cached_method,
     caching::FastUnboundedCache,
     eq_by_big_hash, impl_eq_by_big_hash,
-    name::Name,
     py_types::{use_rust_comp, PyCallable},
     pycall, python_error_exception,
     rearrange_spec::RearrangeSpec,
@@ -239,7 +238,7 @@ impl ModuleSpec {
             )
             .context("rank overflow in module expand")?;
 
-            let rep_name: Option<Name> = out.info().name.map(|x| format!("{} rep_batch", x).into());
+            let rep_name = out.name().map(|x| format!("{} rep_batch", x));
             Rearrange::nrc(out, spec, rep_name)
         } else {
             out
@@ -266,15 +265,12 @@ impl ModuleSpec {
                 Symbol::nrc(
                     s.clone(),
                     self.get_spec_circuit_uuid(),
-                    Some(
-                        format!(
-                            "{}_internal_expand_shape_{}_arg{}",
-                            uuid,
-                            Name::str_maybe_empty(self.circuit.info().name),
-                            i
-                        )
-                        .into(),
-                    ),
+                    Some(format!(
+                        "{}_internal_expand_shape_{}_arg{}",
+                        uuid,
+                        self.circuit.name().unwrap_or(""),
+                        i
+                    )),
                 )
             })
             .collect();
@@ -302,7 +298,7 @@ impl ModuleSpec {
             .map(|x| [x.clone()].into_iter().collect());
         let syms = sym_single_map
             .as_ref()
-            .unwrap_or(self.circuit.info().get_raw_free_symbols());
+            .unwrap_or(&self.circuit.info().free_symbols);
 
         for sym_here in syms {
             if let Some(&my_sym) = my_idents.get(&sym_here.ident()) {
@@ -382,20 +378,19 @@ impl ModuleSpec {
             // (NOTE: these free symbols are now the *expanded* symbols from expand_shape)
             if !x.is_symbol()
                 && x.info()
-                    .get_raw_free_symbols()
+                    .free_symbols
                     .iter()
                     .any(|x| node_mapping.contains_key(&x.info().hash))
             {
-                if let Some(n) = x.info().name {
+                if let Some(n) = x.name() {
                     if n.contains(&clear_names_str) {
                         let name = x.get_autoname().unwrap_or(None);
                         return Some(x.rename(name));
                     } else if name_prefix.is_some() || name_suffix.is_some() {
                         let name = Some(
-                            (name_prefix.as_deref().unwrap_or("").to_owned()
-                                + n.into()
-                                + name_suffix.as_deref().unwrap_or(""))
-                            .into(),
+                            name_prefix.as_deref().unwrap_or("").to_owned()
+                                + n
+                                + name_suffix.as_deref().unwrap_or(""),
                         );
                         return Some(x.rename(name));
                     }
@@ -482,7 +477,7 @@ impl ModuleSpec {
             let new = if let Some(m) = circ.as_module() {
                 let mut removed: BTreeSet<_> = symbols
                     .iter()
-                    .filter(|x| m.info().get_raw_free_symbols().contains(*x))
+                    .filter(|x| m.info().free_symbols.contains(*x))
                     .cloned()
                     .collect();
                 for bound in &m.spec.arg_specs {
@@ -538,7 +533,7 @@ pub fn get_free_symbols(circuit: &Circuit) -> FreeSymbolsWrapper<'_> {
     if let Some(sym) = circuit.as_symbol() {
         FreeSymbolsWrapper::Sym([sym.clone()].into_iter().collect())
     } else {
-        FreeSymbolsWrapper::Other(&circuit.info().get_raw_free_symbols())
+        FreeSymbolsWrapper::Other(&circuit.info().free_symbols)
     }
 }
 
@@ -586,23 +581,19 @@ impl ModuleSpec {
 
     pub fn check_unique_arg_names(&self) -> Result<()> {
         // TODO: maybe cache me (as needed)!
-        if self
-            .arg_specs
-            .iter()
-            .any(|x| x.symbol.info().name.is_none())
-        {
+        if self.arg_specs.iter().any(|x| x.symbol.name().is_none()) {
             bail!(ConstructError::ModuleSomeArgsNamedNone {
                 symbols_named_none: self
                     .arg_specs
                     .iter()
-                    .filter_map(|x| x.symbol.info().name.is_none().then(|| x.symbol.clone()))
+                    .filter_map(|x| x.symbol.name().is_none().then(|| x.symbol.clone()))
                     .collect()
             })
         }
-        let names: Vec<Name> = self
+        let names: Vec<_> = self
             .arg_specs
             .iter()
-            .map(|x| x.symbol.info().name.unwrap())
+            .map(|x| x.symbol.name().unwrap())
             .collect();
         if !is_unique(&names) {
             bail!(ConstructError::ModuleArgsDupNames {
@@ -660,7 +651,7 @@ impl ModuleSpec {
                     arg_spec.symbol = Symbol::new(
                         circuit.info().shape.clone(),
                         arg_spec.symbol.uuid,
-                        arg_spec.symbol.info().name.or(circuit.info().name),
+                        arg_spec.symbol.name_cloned().or(circuit.name_cloned()),
                     );
                     real_arg_specs[i] = Some(arg_spec);
                     return (Some(real_arg_specs[i].as_ref().unwrap().symbol.crc()), true);
@@ -704,7 +695,7 @@ impl ModuleSpec {
     pub fn resize(&self, shapes: Vec<Shape>) -> Result<Self> {
         let arg_specs: Vec<ModuleArgSpec> = zip(&self.arg_specs, shapes)
             .map(|(arg_spec, shape)| ModuleArgSpec {
-                symbol: Symbol::new(shape, arg_spec.symbol.uuid, arg_spec.symbol.info().name),
+                symbol: Symbol::new(shape, arg_spec.symbol.uuid, arg_spec.symbol.name_cloned()),
                 ..arg_spec.clone()
             })
             .collect();
@@ -748,12 +739,11 @@ impl ModuleSpec {
         }
 
         self.map_on_replaced_path_unwrap(|x| {
-            if let Some(n) = x.info().name {
+            if let Some(n) = x.name() {
                 let name = Some(
-                    (prefix.as_deref().unwrap_or("").to_owned()
-                        + n.into()
-                        + suffix.as_deref().unwrap_or(""))
-                    .into(),
+                    prefix.as_deref().unwrap_or("").to_owned()
+                        + n
+                        + suffix.as_deref().unwrap_or(""),
                 );
                 x.rename(name)
             } else {
@@ -836,7 +826,10 @@ impl ModuleArgSpec {
         ban_non_symbolic_size_expand: bool,
     ) -> Self {
         Self {
-            symbol: Symbol::new_with_random_uuid(circuit.info().shape.clone(), circuit.info().name),
+            symbol: Symbol::new_with_random_uuid(
+                circuit.info().shape.clone(),
+                circuit.name_cloned(),
+            ),
             batchable,
             expandable,
             ban_non_symbolic_size_expand,
@@ -869,11 +862,13 @@ pub struct Module {
     pub spec: ModuleSpec,
     substitute_cached_info: Box<CachedCircuitInfo>, // box for size
     info: CachedCircuitInfo,
+    #[pyo3(get)]
+    name: Option<String>,
 }
 
 impl Module {
     #[apply(new_rc_unwrap)]
-    pub fn try_new(nodes: Vec<CircuitRc>, spec: ModuleSpec, name: Option<Name>) -> Result<Self> {
+    pub fn try_new(nodes: Vec<CircuitRc>, spec: ModuleSpec, name: Option<String>) -> Result<Self> {
         // we need to check this here because we substitute modules below where other checks run
         spec.check_no_free_bound_by_nested(&nodes).with_context(|| {
             const MSG: &str = "(free sym bound by nested)";
@@ -902,24 +897,24 @@ impl Module {
         for n in &nodes {
             free_syms.extend(get_free_symbols(&**n).iter().cloned());
         }
-        substitute_cached_info.free_symbols = Some(Arc::new(free_syms));
+        substitute_cached_info.free_symbols = free_syms;
 
         let substitute_cached_info = Box::new(substitute_cached_info);
 
-        let mut out = Self {
+        let out = Self {
             nodes,
             spec,
+            name,
             substitute_cached_info,
             info: Default::default(),
         };
-        out.info.name = name;
         out.initial_init_info()
     }
 
     pub fn new_kwargs(
-        kwargs: &HashMap<Name, CircuitRc>,
+        kwargs: &HashMap<String, CircuitRc>,
         spec: ModuleSpec,
-        name: Option<Name>,
+        name: Option<String>,
     ) -> Result<Self> {
         let mut nodes: Vec<_> = vec![None; spec.arg_specs.len()];
         spec.check_unique_arg_names()?;
@@ -927,18 +922,18 @@ impl Module {
             match spec
                 .arg_specs
                 .iter()
-                .position(|x| x.symbol.info().name.expect("check_unique_arg_names above") == *k)
+                .position(|x| x.symbol.name().expect("check_unique_arg_names above") == k)
             {
                 Some(i) => {
                     nodes[i] = Some(v.clone());
                 }
                 None => {
                     bail!(ConstructError::ModuleUnknownArgument {
-                        argument: *k,
+                        argument: k.clone(),
                         all_module_inputs: spec
                             .arg_specs
                             .iter()
-                            .map(|x| x.symbol.info().name.unwrap())
+                            .map(|x| x.symbol.name_cloned().unwrap())
                             .collect()
                     })
                 }
@@ -951,8 +946,7 @@ impl Module {
                     .zip(spec.arg_specs)
                     .filter_map(|(n, arg_spec)| n.is_none().then(|| arg_spec
                         .symbol
-                        .info()
-                        .name
+                        .name_cloned()
                         .expect("check_unique_arg_names above")))
                     .collect()
             });
@@ -971,7 +965,7 @@ impl Module {
         let out_named = propagate_named_axes(
             stripped,
             (0..self.info().rank())
-                .map(|i| (i as u8, i.to_string().into()))
+                .map(|i| (i as u8, i.to_string()))
                 .collect(),
             true,
         );
@@ -982,9 +976,12 @@ impl Module {
             .collect();
         visit_circuit_unwrap(out_named, |x| {
             if let Some(sym) = x.as_symbol() {
-                if let Some(i) = self.spec.arg_specs.iter().position(|x| {
-                    x.symbol.uuid == sym.uuid && x.symbol.info().name == sym.info().name
-                }) {
+                if let Some(i) = self
+                    .spec
+                    .arg_specs
+                    .iter()
+                    .position(|x| x.symbol.uuid == sym.uuid && x.symbol.name() == sym.name())
+                {
                     for (k, v) in &sym.info().named_axes {
                         result[i][*k as usize] = Some(v.parse::<usize>().unwrap());
                     }
@@ -1005,9 +1002,7 @@ impl CircuitNodeHashItems for Module {
 
 impl CircuitNodeSetNonHashInfo for Module {
     fn set_non_hash_info(&mut self) -> Result<()> {
-        let old_name = self.info().name;
         *self.info_mut() = (*self.substitute_cached_info).clone();
-        self.info_mut().name = old_name;
         Ok(())
     }
 }
@@ -1056,7 +1051,7 @@ impl CircuitNode for Module {
                 circuit: new_spec_circ,
                 arg_specs: self.spec.arg_specs.clone(),
             },
-            self.info().name,
+            self.name.clone(),
         )
     }
 
@@ -1071,7 +1066,7 @@ impl CircuitNode for Module {
             .enumerate()
             .map(|(i, x)| f(i, x))
             .collect::<Result<_>>()?;
-        Self::try_new(nodes, self.spec.clone(), self.info().name)
+        Self::try_new(nodes, self.spec.clone(), self.name_cloned())
     }
 
     fn eval_tensors(
@@ -1088,21 +1083,20 @@ impl CircuitNode for Module {
 impl CircuitNodeAutoName for Module {
     const PRIORITY: OperatorPriority = OperatorPriority::Function {};
 
-    fn auto_name(&self) -> Option<Name> {
-        if self.children().any(|x| x.info().name.is_none()) {
+    fn auto_name(&self) -> Option<String> {
+        if self.children().any(|x| x.name().is_none()) {
             None
         } else {
             Some(
-                (self.spec.circuit.info().name.unwrap().string()
+                self.spec.circuit.name_cloned().unwrap()
                     + "("
                     + &self
                         .nodes
                         .iter()
-                        .map(|node| node.info().name.unwrap().into())
+                        .map(|node| node.name_cloned().unwrap())
                         .collect::<Vec<String>>()
                         .join(", ")
-                    + ")")
-                    .into(),
+                    + ")",
             )
         }
     }
@@ -1114,15 +1108,15 @@ impl Module {
     #[args(name = "None", kwargs = "**")]
     fn new_py(
         spec: ModuleSpec,
-        name: Option<Name>,
-        kwargs: Option<HashMap<Name, CircuitRc>>,
+        name: Option<String>,
+        kwargs: Option<HashMap<String, CircuitRc>>,
     ) -> PyResult<PyClassInitializer<Module>> {
         Ok(Module::new_kwargs(&kwargs.unwrap_or_else(HashMap::default), spec, name)?.into_init())
     }
 
     #[staticmethod]
     #[args(nodes = "*", name = "None")]
-    fn new_flat(spec: ModuleSpec, nodes: Vec<CircuitRc>, name: Option<Name>) -> Result<Self> {
+    fn new_flat(spec: ModuleSpec, nodes: Vec<CircuitRc>, name: Option<String>) -> Result<Self> {
         Self::try_new(nodes, spec, name)
     }
 
@@ -1154,7 +1148,7 @@ impl Module {
         Ok(Self::new(
             self.nodes.clone(),
             self.spec.rename_on_replaced_path(prefix, suffix),
-            self.info().name,
+            self.name_cloned(),
         ))
     }
 
@@ -1162,7 +1156,7 @@ impl Module {
         Ok(Self::new(
             self.nodes.clone(),
             self.spec.py_map_on_replaced_path(f)?,
-            self.info().name,
+            self.name_cloned(),
         ))
     }
 
@@ -1202,7 +1196,7 @@ impl Module {
                         .collect(),
                 )
                 .expect("constructor should ensure this works"),
-            self.info().name,
+            self.name_cloned(),
         ))
     }
 
@@ -1213,7 +1207,7 @@ impl Module {
             self.spec
                 .resize(self.nodes.iter().map(|x| x.shape().clone()).collect())
                 .expect("constructor should ensure this works"),
-            self.info().name,
+            self.name_cloned(),
         )
     }
 
@@ -1244,7 +1238,7 @@ impl Module {
 
 impl Module {
     pub fn substitute_self_name_prefix(&self) -> CircuitRc {
-        self.substitute(self.info().name.map(|x| x.string() + "."), None)
+        self.substitute(self.name_cloned().map(|x| x + "."), None)
     }
 
     pub fn substitute(
@@ -1255,7 +1249,7 @@ impl Module {
         self.spec
             .substitute(&self.nodes, name_prefix, name_suffix)
             .expect("constructor should ensure this works")
-            .rename(self.info().name)
+            .rename(self.name_cloned())
     }
 
     fn get_prefix(
@@ -1273,7 +1267,7 @@ impl Module {
                 )
             }
 
-            Ok(self.info().name.map(|x| x.string() + "."))
+            Ok(self.name_cloned().map(|x| x + "."))
         } else {
             Ok(name_prefix)
         }
@@ -1313,7 +1307,7 @@ impl AllModuleSubstituter {
             // Not clear if this would be slower or faster overall.
             .substitute(&nodes, None, None)
             .expect("module constructor should ensure this works")
-            .rename(m.info().name)
+            .rename(m.name_cloned())
         } else {
             circuit.map_children_unwrap_idxs(|i| new_children[i].clone())
         }
@@ -1385,7 +1379,7 @@ pub fn has_free_sym(circuit: &CircuitRc, sym: &Symbol) -> bool {
     if let Some(other) = circuit.as_symbol() {
         other == sym
     } else {
-        circuit.info().get_raw_free_symbols().contains(sym)
+        circuit.info().free_symbols.contains(sym)
     }
 }
 
@@ -1397,7 +1391,7 @@ pub fn is_intersecting_free_syms_iter<'a>(
         syms.into_iter().any(|x| x == other)
     } else {
         syms.into_iter()
-            .any(|x| circuit.info().get_raw_free_symbols().contains(x))
+            .any(|x| circuit.info().free_symbols.contains(x))
     }
 }
 
@@ -1405,7 +1399,7 @@ pub fn is_intersecting_free_syms(circuit: &CircuitRc, syms: &IndexSet<Symbol>) -
     if let Some(other) = circuit.as_symbol() {
         syms.contains(other)
     } else {
-        !circuit.info().get_raw_free_symbols().is_disjoint(syms)
+        !circuit.info().free_symbols.is_disjoint(syms)
     }
 }
 
