@@ -14,7 +14,6 @@ use itertools::Itertools;
 use macro_rules_attribute::apply;
 use pyo3::{exceptions::PyValueError, prelude::*};
 use rr_util::{
-    compact_data::U8Set,
     filter_by_variant,
     opt_einsum::optimize_einsum_spec_cached,
     py_types::PY_CIRCUIT_ITEMS,
@@ -734,7 +733,7 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     if !did_anything {
         return None;
     }
-    let ints_in_new_args: U8Set = new_args
+    let ints_in_new_args: HashSet<u8> = new_args
         .iter()
         .flat_map(|(_node, ints)| ints.as_slice())
         .cloned()
@@ -744,9 +743,9 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     let shape_map = einsum.shape_map().unwrap();
     let reduced_axes = einsum.reduced_axes();
     let scalar_mul: f64 = reduced_axes
-        .into_iter()
-        .filter(|i| !ints_in_new_args.contains(*i))
-        .map(|i| shape_map[&i] as f64)
+        .iter()
+        .filter(|i| !ints_in_new_args.contains(i))
+        .map(|i| shape_map[i] as f64)
         .product();
     if scalar_mul != 1.0 {
         new_args.push((Scalar::new(scalar_mul, sv![], None).rc(), tu8v![]));
@@ -755,7 +754,7 @@ pub fn einsum_pull_removable_axes(einsum: &Einsum) -> Option<CircuitRc> {
     let new_out_ints: EinsumAxes = einsum
         .out_axes
         .iter()
-        .filter(|i| ints_in_new_args.contains(**i))
+        .filter(|i| ints_in_new_args.contains(i))
         .copied()
         .collect();
     let new_einsum = Einsum::nrc(new_args, new_out_ints.clone(), einsum.name_cloned());
