@@ -25,7 +25,7 @@ use rustc_hash::FxHashMap as HashMap;
 use crate::{
     canonicalize::deep_canonicalize,
     compiler_heuristics::deep_maybe_distribute,
-    compiler_strip::strip_names_and_tags,
+    compiler_strip::{remove_autotags, strip_names},
     deep_rewrite::{
         compiler_simp, deep_heuristic_nest_adds, deep_optimize_einsums, deep_push_down_index_raw,
     },
@@ -81,8 +81,6 @@ pub struct OptimizationSettings {
     pub optimization_parallelism: Option<usize>,
     #[pyo3(get, set)]
     pub device_dtype: Option<TorchDeviceDtype>,
-    #[pyo3(get, set)]
-    pub keep_all_names: bool,
 }
 
 impl Default for OptimizationSettings {
@@ -107,7 +105,6 @@ impl Default for OptimizationSettings {
             save_circ: false,
             optimization_parallelism: Some(8),
             device_dtype: None,
-            keep_all_names: false,
         }
     }
 }
@@ -134,8 +131,7 @@ impl OptimizationSettings {
         log_slow_einsums = "OptimizationSettings::default().log_slow_einsums",
         save_circ = "OptimizationSettings::default().save_circ",
         optimization_parallelism = "None",
-        device_dtype = "None",
-        keep_all_names = "OptimizationSettings::default().keep_all_names"
+        device_dtype = "None"
     )]
     fn new(
         verbose: usize,
@@ -157,7 +153,6 @@ impl OptimizationSettings {
         save_circ: bool,
         optimization_parallelism: Option<usize>,
         device_dtype: Option<TorchDeviceDtype>,
-        keep_all_names: bool,
     ) -> Self {
         Self {
             verbose,
@@ -179,7 +174,6 @@ impl OptimizationSettings {
             save_circ,
             optimization_parallelism,
             device_dtype,
-            keep_all_names,
         }
     }
 }
@@ -245,11 +239,8 @@ pub fn optimize_circuit(
     context: &mut OptimizationContext,
 ) -> Result<CircuitRc> {
     let print_timings = context.settings.verbose >= 2;
-    let circuit = if !context.settings.keep_all_names {
-        timed!(strip_names_and_tags(circuit, context), 10, print_timings)
-    } else {
-        circuit
-    };
+    let circuit = timed!(strip_names(circuit, context, true), 10, print_timings);
+    let circuit = remove_autotags(circuit);
     let circuit = conform_all_modules(circuit);
     let circuit = inline_single_callsite_modules(circuit);
     if is_python_running() && any_children_with_symbolic_sizes(circuit.clone()) {
