@@ -1,12 +1,14 @@
 use circuit_base::{
     evaluate_device_dtype,
     generalfunction::{SpecTrait, OFFICIAL_GENERALFUNCTION_INVERSES},
-    Circuit, CircuitNode, CircuitRc, GeneralFunction, Scalar,
+    Array, Circuit, CircuitNode, CircuitRc, GeneralFunction, PyCircuitBase, Scalar,
 };
 use pyo3::prelude::*;
 use rr_util::{
-    sv,
-    tensor_util::{TorchDeviceDtypeOp, TorchDtype},
+    py_types::scalar_to_tensor,
+    pycall, sv,
+    symbolic_size::SymbolicSizeProduct,
+    tensor_util::{TorchDeviceDtype, TorchDeviceDtypeOp, TorchDtype},
 };
 #[pyfunction]
 pub fn generalfunction_merge_inverses(circuit: &GeneralFunction) -> Option<CircuitRc> {
@@ -44,6 +46,24 @@ pub fn generalfunction_special_case_simplification(circuit: &GeneralFunction) ->
                     ));
                 }
             }
+            "last_dim_size" => {
+                let last_dim_maybe_symbolic =
+                    circuit.nodes[0].info().shape[circuit.nodes[0].info().rank() - 1];
+                if !SymbolicSizeProduct::has_symbolic(last_dim_maybe_symbolic) {
+                    return Some(Array::nrc(
+                        scalar_to_tensor(
+                            last_dim_maybe_symbolic as f64,
+                            circuit.nodes[0].info().shape[..circuit.nodes[0].info().rank() - 1]
+                                .iter()
+                                .cloned()
+                                .collect(),
+                            circuit.info().device_dtype.unwrap_or_defaults(),
+                        )
+                        .unwrap(),
+                        None,
+                    ));
+                }
+            }
             _ => {}
         }
     }
