@@ -777,7 +777,9 @@ impl TensorIndex {
                         )
                     } else {
                         save_tensor((**tensor).clone(), false)?;
-                        "t".to_owned() + &base16::encode_lower(&tensor.hash().unwrap())[..10]
+                        "t".to_owned()
+                            + &base16::encode_lower(&tensor.hash().unwrap())[..10]
+                            + &format!(" [{}]", tensor.shape()[0])
                     }
                 }
             });
@@ -810,11 +812,13 @@ impl TensorIndex {
                         return Ok(TensorAxisIndex::Single(int));
                     }
                     if let Some(no_t) = x.strip_prefix('t') {
+                        let no_t_pre_bracket = no_t.split("[").next().unwrap().trim();
                         return tensor_cache
                             .as_mut()
-                            .map(|tc| tc.get_tensor(no_t.to_owned()))
+                            .map(|tc| tc.get_tensor(no_t_pre_bracket.to_owned()))
                             .unwrap_or_else(|| {
-                                get_tensor_prefix(no_t).context("tensor hash from string")
+                                get_tensor_prefix(no_t_pre_bracket)
+                                    .context("tensor hash from string")
                             })
                             .and_then(|x| {
                                 Ok(TensorAxisIndex::Tensor(
