@@ -2,8 +2,7 @@
 import uuid
 
 import torch
-
-from rust_circuit import Einsum, Scalar, Array, Symbol, Add, Rearrange, RearrangeSpec, Concat, Index
+from rust_circuit import Add, Array, Concat, Einsum, Index, Rearrange, RearrangeSpec, Scalar, Symbol
 
 # %%
 e = Einsum.from_einsum_string("a->", Scalar(0.2, (2,)))
